#include <algorithm>
#include <array>
#include <cmath>
#include <cstdint>
#include <iostream>
#include <iomanip>
#include <memory>
#include <numeric>
#include <set>

struct Node {
    Node(int64_t l, int64_t r): l(l), r(r), minArea(r - l) {}

    int64_t val() const {
        return minVal + lazyVal;
    }

    void makeChildren() {
        if (children[0]) return;
        int64_t m = std::midpoint(l, r);
        children[0] = std::make_unique<Node>(l, m);
        children[1] = std::make_unique<Node>(m, r);
    }

    void pull() {
        if (!children[0]) return;

        if (children[0]->val() < children[1]->val()) {
            minVal = children[0]->val() + lazyVal;
            minArea = children[0]->minArea;
        } else if (children[0]->val() > children[1]->val()) {
            minVal = children[1]->val() + lazyVal;
            minArea = children[1]->minArea;
        } else {
            minVal = children[0]->val() + lazyVal;
            minArea = children[0]->minArea + children[1]->minArea;
        }
    }

    void update(int64_t ql, int64_t qr, int64_t delta) {
        if (r <= ql || qr <= l) {
            return;
        }
        if (ql <= l && r <= qr) {
            lazyVal += delta;
            pull();
            return;
        }

        makeChildren();

        for (auto& c : children) {
            c->update(ql, qr, delta);
        }

        pull();
    }

    int64_t l = 0, r = 0;
    int64_t minVal = 0;
    int64_t minArea = 0;
    int64_t lazyVal = 0;
    std::array<std::unique_ptr<Node>, 2> children{{nullptr, nullptr}};
};

int main() {
    int64_t q, w;
    std::cin >> q >> w;

    double sqrt2 = std::sqrt(2.0);

    Node root(0, w);
    std::set<std::array<int64_t, 2>> peaks;
    while (q-- > 0) {
        int64_t x, y;
        std::cin >> x >> y;
        auto [it, isNew] = peaks.insert({x, y});

        int64_t delta = isNew ? 1 : -1;
        if (!isNew)
            peaks.erase(it);

        int64_t l = std::max<int64_t>(0, x - y);
        int64_t r = std::min<int64_t>(w, x + y);

        root.update(l, r, delta);

        int64_t base = w - (root.val() == 0 ? root.minArea : 0);

        std::cout << std::fixed << std::setprecision(20) << base * sqrt2 << '\n';
    }

    return 0;
}
