#include <algorithm>
#include <array>
#include <cmath>
#include <iomanip>
#include <iostream>
#include <set>
#include <vector>

using namespace std;

struct seg {
  vector<int> rt, weight;
  vector<int> cnt;
  int SZ = 1;
  seg(const vector<int>& weights) {
    int n = weights.size();
    while(SZ < n) SZ *= 2;
    rt.resize(2*SZ);
    cnt.resize(2*SZ);
    weight.resize(2*SZ);
    for(int i = 0; i < weights.size(); i++) weight[i+SZ] = weights[i];
    for(int i = SZ-1; i > 0; i--) weight[i] = weight[2*i] + weight[2*i+1];
  }
  void apply(int idx, int sz, int val) {
    cnt[idx] += val;
    if(cnt[idx] > 0) rt[idx] = weight[idx];
    else if(sz == 1) rt[idx] = 0;
    else rt[idx] = rt[2*idx] + rt[2*idx+1];
  }
  void build(int idx) {
    int csz = 1;
    while(idx > 0) {
      if(cnt[idx] > 0) rt[idx] = weight[idx];
      else if(csz == 1) rt[idx] = 0;
      else rt[idx] = rt[2*idx] + rt[2*idx+1];
      idx /= 2;
      csz *= 2;
    }
  }
  void upd(int lhs, int rhs, int val) {
    lhs += SZ;
    rhs += SZ;
    int olhs = lhs;
    int orhs = rhs;
    int csz = 1;
    while(lhs <= rhs) {
      if(lhs%2) apply(lhs++, csz, val);
      if(rhs%2==0) apply(rhs--, csz, val);
      lhs /= 2;
      rhs /= 2;
      csz *= 2;
    }
    build(olhs);
    build(orhs);
  }
};

const double S = sqrt(2);

int main() {
  int n, w;
  cin >> n >> w;
  vector<array<int, 2>> v(n);
  vector<int> xs;
  for(auto& x: v) {
    cin >> x[0] >> x[1];
    int lhs = max(0, x[0] - x[1]);
    int rhs = min(w, x[0] + x[1]);
    xs.push_back(lhs);
    xs.push_back(rhs);
  }
  sort(xs.begin(), xs.end());
  xs.erase(unique(xs.begin(), xs.end()), xs.end());
  vector<int> weights;
  for(int i = 0; i + 1 < xs.size(); i++) weights.push_back(xs[i+1] - xs[i]);
  seg s(weights);
  set<array<int, 2>> pts;
  cout << fixed << setprecision(39);
  for(auto& x: v) {
    int scale = 1;
    if(pts.erase(x)) scale = -1;
    else pts.insert(x);
    int lhs = max(0, x[0] - x[1]);
    int rhs = min(w, x[0] + x[1]);
    lhs = lower_bound(xs.begin(), xs.end(), lhs) - xs.begin();
    rhs = lower_bound(xs.begin(), xs.end(), rhs) - xs.begin() - 1;
    s.upd(lhs, rhs, scale);
    cout << S * s.rt[1] << "\n";
  }
}